//-----------------------------LICENSE NOTICE------------------------------------
//   ____        _           _       _____      _                                
//  |  _ \      | |         ( )     |  __ \    | |                               
//  | |_) | __ _| |__   __ _|/ ___  | |__) |_ _| | __ _  ___ ___                 
//  |  _ < / _` | '_ \ / _` | / __| |  ___/ _` | |/ _` |/ __/ _ |                
//  | |_) | (_| | |_) | (_| | \__ \ | |  | (_| | | (_| | (_|  __/                
//  |____/ \__,_|_.__/ \__,_| |___/ |_|   \__,_|_|\__,_|\___\___|                
//
//  Code & Gfx Copyright (C) 2017 Rafael Castillo (@Azicuetano)
//  Music & Fx Copyright (C) 2017 John McKlain (@elmcklain)
//  This file is part of BABA'S PALACE: an Amstrad CPC Game made with CPCTelera
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include <cpctelera.h>
#include <stdio.h>

#include "menu.h"
#include "game.h"

#include "../keyboard/keyboard.h"

#include "../globals.h"
#include "../text/text.h"
#include "../utils/uitoa.h"

#include "../sprites/menu-star.h"
#include "../sprites/menu-mountains.h"
#include "../sprites/palace_cpcretrodev.h"

#include "../sprites/char.h"
#include "../audio/music.h"


const u8 game_palette[7][16] = {

    { 0x55, 0x54, 0x44, 0x55, 0x5c, 0x58, 0x5d, 0x4c, 0x56, 0x5e, 0x40, 0x5f, 0x4e, 0x47, 0x4a, 0x4b },
    { 0x55, 0x54, 0x44, 0x55, 0x5c, 0x58, 0x5d, 0x4c, 0x56, 0x5e, 0x40, 0x5f, 0x4e, 0x47, 0x4a, 0x4a },
    { 0x55, 0x54, 0x44, 0x55, 0x5c, 0x58, 0x5d, 0x4c, 0x56, 0x5e, 0x5e, 0x57, 0x4e, 0x4e, 0x59, 0x59 },
    { 0x55, 0x54, 0x44, 0x55, 0x5c, 0x58, 0x5d, 0x5c, 0x58, 0x5d, 0x5d, 0x5d, 0x4c, 0x4c, 0x46, 0x46 },
    { 0x44, 0x54, 0x54, 0x44, 0x44, 0x44, 0x44, 0x44, 0x44, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x55, 0x55 }, 
    { 0x44, 0x54, 0x54, 0x44, 0x54, 0x54, 0x54, 0x54, 0x54, 0x44, 0x44, 0x44, 0x44, 0x44, 0x44, 0x44 },
    { 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54 }
};

const u8 ending_palette[16] = { 0x5b, 0x5b, 0x5b, 0x57, 0x44, 0x55, 0x5d, 0x4c, 0x56, 0x5e, 0x40, 0x5f, 0x4e, 0x47, 0x4a, 0x4b };

// FX Sounds
// index = (number of the instrument - 1). Data = { volume, note }
const u8 fx[11][2] = {

    { 0, 0 }, // no instrument 0
    { 13, 48 }, // JUMP_TO
    { 13, 48 }, // JUMP_FROM
    { 15, 38 }, // SWAP_HERO
    { 15, 62 }, // EXPLOSION
    { 14, 41 }, // PUSH
    { 14, 48 }, // BREAK_SOFTWALL
    { 14, 36 }, // LANDING
    { 15, 60 }, // OPEN_MENU
    { 11, 67 }, // SELECT
    { 11, 67 }  // CLICK
};

u8 g_lakeColor = 0;
u8 g_skyColor = 0;

u8 g_nLevel = 0;                // Actual level.
u8 g_menuSelectedOption = 0;    // Manage menu selections.
u8 g_godmode = 0;               // God Mode.


u8 kk = 0; // To get Raster sincronized, I need to reestar this var every time menu screen is initialized.


// Stars position (the first 5 are big stars).
const u8 stars[16][2] = { 
    {72,07}, {74,23}, {8,26}, {16,66}, {66, 90}, 
    {6,9}, {2,56}, {8,50}, {4,70}, {16,3}, {20,109}, {32,102}, {64,18}, {68,25}, {62,40}, {58,86}
  };


void cpct_scanKeyboard_if();
void cpct_removeInterruptHandler();


////////////////////////////////////////////////////////////////////////////////
//
//    FX SOUNDS MANAGEMENT
//
//    This works by making use of global constants in globals.h that identify each fx sound you want to play.
//    Examples of usage: playFX(JUMP_TO); playFX(EXPLOSION); ...

void playFX(u8 fxType) {
    u8 volume, note;

    volume = fx[fxType][0];
    note =   fx[fxType][1];

    cpct_akp_SFXPlay(fxType, volume, note, 0, 0, AY_CHANNEL_A);
}

////////////////////////////////////////////////////////////////////////////////
//
//    INTERRUPT MANAGEMENT
//
//

void interruptHandler() {
    // Statics variable to be preserved from call to call

    static u8 k;        // Master interrupt control.
    static u8 i;        // Blinking stars index
    static u8* pvideo;

    //
    // MUSIC MANAGEMENT (Awesome Music By McKlain!!!)
    // 
    if (++kk == 6) {
        cpct_scanKeyboard_if();
        cpct_akp_musicPlay();
        kk = 0;
    }


    if(kk==1) cpct_setPALColour (16, g_skyColor);
    if(kk==5) cpct_setPALColour (16, g_lakeColor);



    //
    // BLINKING STARS
    //

    if(g_skyColor!=0x5b) // Not in ending screen...
    {
      switch (k) {

      case 0:   // STAR GO TO BLINK FRAME

        // Choose blinking star
        i=cpct_getRandom_lcg_u8(0);
        // We have 16 stars, so I divided the 256 posible value by 16 to obtain a random number from 0 to 15. 
        i=i>>4;

        pvideo = cpct_getScreenPtr(CPCT_VMEM_START, stars[i][0], stars[i][1]);

        if (i<5) // First 5 stars are big stars
        {
          cpct_drawSprite(gfx_menu_star_1, pvideo, GFX_MENU_STAR_1_W, GFX_MENU_STAR_1_H);
        }
        else {
          // Define a blink star of 1 pixel.
          // precalculated pattern8: cpct_px2byteM0(14,2) = 0x2e;
          *pvideo = 0x2e;

        }
        break;


      case 24:   // STAR TURN TO NORMAL FRAME
        
        if (i<5) // First 5 stars are big stars
        {
          cpct_drawSprite(gfx_menu_star_0, pvideo, GFX_MENU_STAR_0_W, GFX_MENU_STAR_0_H);
        } 
        else {
          // Define a Pastel Blue color star of 1 pixel.
          // precalculated patern8: cpct_px2byteM0(11,2) = 0x8e;
          // pvideo = cpct_getScreenPtr(CPCT_VMEM_START, stars[i][0], stars[i][1]);
          *pvideo = 0x8e;
        }

        // Breaking the period of blinking. K<=20 are protected.
        k+= cpct_rand()-25;
        break;
      }
      ++k;
    }

}


// direction = [IN|OUT] = [0|6] (defined in globals.h). Examples: fade(IN); fade(OUT);
void fade(u8 direction){
    u8 i, pal;

    for(i=0;i<=OUT;i++){                        
        pal=i;
        if(direction==IN) pal=OUT-i;        // invert direction to perform a fadeIn.
        
        wait_n_VSYNCs(2);
              
        if (pal==0) {
            g_skyColor=0x44;
            g_lakeColor=0x55;
        } else {
            g_skyColor=0x54;
            g_lakeColor=0x54;
        }

        cpct_setPalette(game_palette[pal], 16);
    }

}




////////////////////////////////////////////////////////////////////////////////
//
//    ENCRIPT & DECRIPT LEVEL CODES
//
//

// real universe = "ABCEFHJKLPRSTXYZ";
const u8 universe[] =   "HZSBPAEJLXYKTRCF0123456789";
const cpct_keyID keyUniverse[] = {Key_H,Key_Z,Key_S,Key_B,Key_P,Key_A,Key_E,Key_J,Key_L,Key_X,Key_Y,Key_K,Key_T,Key_R,Key_C,Key_F, Key_0,Key_1,Key_2,Key_3,Key_4,Key_5,Key_6,Key_7,Key_8,Key_9};

void getCodeFromLevel(u8 *levelCode) {
    u16 code; //*** Atention: 1=0100, 255=FF00, 256=0001...

    code = KEY * g_nLevel;

    // I need to alternate 2 bytes in u16 to work in the right order.
    levelCode[0] = universe[ cpct_get4Bits (&code, 2) ];
    levelCode[1] = universe[ cpct_get4Bits (&code, 3) ];
    levelCode[2] = universe[ cpct_get4Bits (&code, 0) ];
    levelCode[3] = universe[ cpct_get4Bits (&code, 1) ];
}

u8 getIndexFromUniverse(u8 letter){
    u8 i=0;

    for(i=0;i<16;i++)
    if (universe[i]==letter) break;

    return i;
}

u8 getLevelFromCode(u8 *levelCode) {
    u16 code; //*** Atention: 1=0100, 255=FF00, 256=0001...

    //The next line is useless, forget it, your mind will go blank in 3, 2, 1 ... Now!
    if(levelCode[0]=='C' && levelCode[1]=='P' && levelCode[2]=='C' && levelCode[3]=='T') g_godmode=YES;

    // I need to alternate 2 bytes in u16 to work in the right order.
    cpct_set4Bits (&code, getIndexFromUniverse( levelCode[2] ), 0); // 
    cpct_set4Bits (&code, getIndexFromUniverse( levelCode[3] ), 1); // 
    cpct_set4Bits (&code, getIndexFromUniverse( levelCode[0] ), 2); // 
    cpct_set4Bits (&code, getIndexFromUniverse( levelCode[1] ), 3); // 

    if( (code%KEY) == 0) return code/KEY; // if validation is ok, return the number of level.
    else return 0;                        // return error.
}

void getTextLevelCode(u8 *textlevelCode) {
    char cvalue[6];  // 5 are the maximus digits suported by uitoa().

    //REMEMBER: textlevelCode[] = "Code Level 100:";

    uitoa(g_nLevel, cvalue);

    if (g_nLevel<10) {
        textlevelCode[11]='0';
        textlevelCode[12]=cvalue[0];
        textlevelCode[13]=':';
        textlevelCode[14]=' ';
    } else if (g_nLevel<100) {
        textlevelCode[11]=cvalue[0];
        textlevelCode[12]=cvalue[1];
        textlevelCode[13]=':';
        textlevelCode[14]=' ';
    }

}



////////////////////////////////////////////////////////////////////////////////
//
//    MANAGE MENU OPTIONS
//
//


cpct_keyID redefineKey(u8 *text){
    
    cpct_keyID key;
  
    // clear menu opctions
    clearOptions();

    drawText("Press a key to", 27, 45, COLORTXT_YELLOW, NORMALHEIGHT, TRANSPARENT);
    drawText(text, 34, 60, COLORTXT_YELLOW, DOUBLEHEIGHT, TRANSPARENT);
    
    key = waitAKey();
    waitKeyUp(key);
    playFX(CLICK);
    
    return key;
    
}

void checkKeyboard() {

    cpct_keyID key;
    u8 i=0, ii, level, universeLimit=16;
    char keyChar[]="_", levelCode[]="0123";
    u8 *messages[2] = {"Invalid code! :(", "The code is ok!!"};



    if ( cpct_isKeyPressed(Key_1) ) {
    //
    // 1. PLAY WITH JOYSTICK;
    //

        playFX(SELECT);
        waitKeyUp(Key_1);

        keys.left  = Joy0_Left;
        keys.right = Joy0_Right;
        keys.up    = Joy0_Up;
        keys.down  = Joy0_Down;
        keys.swap  = Joy0_Fire1;

        g_menuSelectedOption=1;
    } 
    else if ( cpct_isKeyPressed(Key_2) ) {
        //
        // 2. PLAY WITH KEYBOARD
        //
        playFX(SELECT);
        waitKeyUp(Key_2);
        // The default keys are defined on menuInit() function.

        keys.left  = userKeys.left;
        keys.right = userKeys.right;
        keys.up    = userKeys.up;
        keys.down  = userKeys.down;
        keys.swap  = userKeys.swap;

        g_menuSelectedOption=2;

    }
    else if ( cpct_isKeyPressed(Key_3) ) {
        //
        // 3. REDEFINE KEYS
        //

        playFX(SELECT);
        waitKeyUp(Key_3);

        // OPQA
        //
        // According to the results of the referendum celebrated in RetroZaragoza the 1 of October of 2017, published by @octopusjig.
        // More info: https://twitter.com/octopusjig/status/914488186545561600
        //
        // Votes cast: 134
        // null votes: 1
        // QAOP: 65
        // OPQA: 68 <-- WINNER!!! :)

        userKeys.left  = redefineKey(" LEFT");
        userKeys.right = redefineKey(" RIGHT");
        userKeys.up    = redefineKey("  UP");
        userKeys.down  = redefineKey(" DOWN");
        userKeys.swap  = redefineKey("ACTION");

        drawMainOptions();
    }
    else if ( cpct_isKeyPressed(Key_4) ) {
        //
        // 4. LOAD LEVEL
        //

        playFX(SELECT);
        waitKeyUp(Key_4);

        // clear menu opctions
        clearOptions();
        drawText("Enter level code", 25, 45, COLORTXT_YELLOW, NORMALHEIGHT, TRANSPARENT);
        drawText("_ _ _ _", 33, 60, COLORTXT_YELLOW, DOUBLEHEIGHT, TRANSPARENT);


        if( g_godmode==YES) universeLimit=26;
        
        do {
            key = waitAKey();
            waitKeyUp(key);

            ii=0;
            do {
                if(key==keyUniverse[ii]) {
                    keyChar[0]=universe[ii];
                    break;
                }

            } while(ii++<universeLimit);


            if(keyChar[0]!='_') {
                playFX(CLICK);

                levelCode[i]=keyChar[0];

                // Clear _ space to put char on screen.
                cpct_drawSolidBox ((u16*)(0xD2F1+(i*4)), 0x0C, 4, 4);
                // Draw char related to pressed key.
                drawText(keyChar, 33+(i*4), 60, COLORTXT_WHITE, DOUBLEHEIGHT, TRANSPARENT);

                // reset keyChar for detect a new valid Key.
                keyChar[0]='_';
                ++i;
            }
        } while(i<4);

        // Get the nºLevel from code. 0=wrong code.
        if( g_godmode==NO) level=getLevelFromCode(levelCode);
        else {
            // GODMODE
            level=(levelCode[1]-'0')*100 + (levelCode[2]-'0')*10 + (levelCode[3]-'0');
            if(level>100) level=0;  // to avoid worng codes...
        }

        if(level==0) 
            i=0;   // Index to invalid code message.
        else {
            i=1;   // Index to valid code message.
            g_nLevel = level;
        }

        // Precalculated pvideo = 0xe9a9
        cpct_drawSolidBox ((u16*)0xe9a9, 0x0C, 32, 9);
        wait_n_VSYNCs(10); // wait 0.2 seconds.
        drawText(messages[i], 25, 45, COLORTXT_YELLOW, NORMALHEIGHT, TRANSPARENT);
        wait_n_VSYNCs(100); // wait 2 seconds.

        drawMainOptions();
    }
}



////////////////////////////////////////////////////////////////////////////////
//
//    INITIALIZATION
//
//

void menuInit() {
    
    cpct_setPalette(game_palette[6], 16);
    cpct_setPALColour (16, 0x54);
    g_skyColor=0x54;
    g_lakeColor=0x54;

    cpct_waitVSYNC ();
    cpct_clearScreen(0x0c0c);
    cpct_waitVSYNC ();


    // Music menú
    cpct_akp_musicInit(G_music_menu);
    // FX
    cpct_akp_SFXInit (G_fx);

    // If keys never was redefined...
    if(!userKeys.up){
        userKeys.left  = Key_CursorLeft;
        userKeys.right = Key_CursorRight;
        userKeys.up    = Key_CursorUp;
        userKeys.down  = Key_CursorDown;
        userKeys.swap  = Key_Space;
        keys.menu  = Key_Esc;   // this key can't be redefined by user.
    }
    
    // Real levels goes from 1 to 100.
    if(g_nLevel==0) g_nLevel=1; // Mandatory to init play.
  
    kk=0;
}


////////////////////////////////////////////////////////////////////////////////
//
//    MENU SCREEN
//
//

void clearOptions(){

    cpct_waitVSYNC();
    cpct_drawSolidBox ((u16*)0xc906, 0x0C, 40, 60);
}


void drawCodeLevel(u8 posX, u8 posY){
    // initialization values
    u8 offsetX=29;
    char levelCode[] = "0000", textlevelCode[] = "Code Level 100:";

    // If level > 1, draw info about Code Level for player.
    if(g_nLevel>1 && g_nLevel<101) {
        getTextLevelCode(textlevelCode);
        drawText(textlevelCode, posX, posY, COLORTXT_MAUVE, NORMALHEIGHT, TRANSPARENT);

        if(g_nLevel==100) offsetX+=2;
        getCodeFromLevel(levelCode);
        drawText(levelCode, posX+offsetX, posY, COLORTXT_BLUE, NORMALHEIGHT, TRANSPARENT);
    }
}

void drawMainOptions(){
  
    clearOptions();

    drawCodeLevel(22, 25);
    drawText("1. Joystick", 28, 45,   COLORTXT_WHITE, NORMALHEIGHT, TRANSPARENT);
    drawText("2. Keyboard", 28, 55,   COLORTXT_WHITE, NORMALHEIGHT, TRANSPARENT);
    drawText("3. Redefine", 28, 65,   COLORTXT_WHITE, NORMALHEIGHT, TRANSPARENT);
    drawText("4. Load level", 28, 75, COLORTXT_WHITE, NORMALHEIGHT, TRANSPARENT);
}


void drawLandcape() {

    //
    // DRAW PALACE
    //
    // precalculated: pvideo = cpct_getScreenPtr(CPCT_VMEM_START, 0, 122-3);
    cpct_drawSprite(gfx_menu_mountains, (u16*)64608, GFX_MENU_MOUNTAINS_W, GFX_MENU_MOUNTAINS_H);
    // precalculated: pvideo = cpct_getScreenPtr(CPCT_VMEM_START, 11, 122-3);
    cpct_drawSprite(gfx_menu_mountains, (u16*)64619, GFX_MENU_MOUNTAINS_W, GFX_MENU_MOUNTAINS_H);


    // -- Palace part 1
    // precalculated: pvideo = cpct_getScreenPtr(CPCT_VMEM_START, 22, 122);
    cpct_drawSprite(gfx_palace1_cpcretrodev, (u16*)54470, GFX_PALACE1_CPCRETRODEV_W, GFX_PALACE1_CPCRETRODEV_H);
    cpct_hflipSpriteM0 (GFX_PALACE1_CPCRETRODEV_W, GFX_PALACE1_CPCRETRODEV_H, gfx_palace1_cpcretrodev);
    // precalculated: pvideo = cpct_getScreenPtr(CPCT_VMEM_START, 40, 122);
    cpct_drawSprite(gfx_palace1_cpcretrodev, (u16*)54488, GFX_PALACE1_CPCRETRODEV_W, GFX_PALACE1_CPCRETRODEV_H);
    cpct_hflipSpriteM0 (GFX_PALACE1_CPCRETRODEV_W, GFX_PALACE1_CPCRETRODEV_H, gfx_palace1_cpcretrodev);
    // -- Palace part 2
    // precalculated: pvideo = cpct_getScreenPtr(CPCT_VMEM_START, 34, 104);
    cpct_drawSprite(gfx_palace2_cpcretrodev, (u16*)50226, GFX_PALACE2_CPCRETRODEV_W, GFX_PALACE2_CPCRETRODEV_H);
    cpct_hflipSpriteM0 (GFX_PALACE2_CPCRETRODEV_W, GFX_PALACE2_CPCRETRODEV_H, gfx_palace2_cpcretrodev);
    // precalculated: pvideo = cpct_getScreenPtr(CPCT_VMEM_START, 40, 104);
    cpct_drawSprite(gfx_palace2_cpcretrodev, (u16*)50232, GFX_PALACE2_CPCRETRODEV_W, GFX_PALACE2_CPCRETRODEV_H);
    cpct_hflipSpriteM0 (GFX_PALACE2_CPCRETRODEV_W, GFX_PALACE2_CPCRETRODEV_H, gfx_palace2_cpcretrodev);
    // -- Palace part 3
    // precalculated: pvideo = cpct_getScreenPtr(CPCT_VMEM_START, 37, 93);
    cpct_drawSprite(gfx_palace3_cpcretrodev, (u16*)60309, GFX_PALACE3_CPCRETRODEV_W, GFX_PALACE3_CPCRETRODEV_H);


    // precalculated: pvideo = cpct_getScreenPtr(CPCT_VMEM_START, 58, 122-3);
    cpct_drawSprite(gfx_menu_mountains, (u16*)64666, GFX_MENU_MOUNTAINS_W, GFX_MENU_MOUNTAINS_H);
    // precalculated: pvideo = cpct_getScreenPtr(CPCT_VMEM_START, 69, 122-3);
    cpct_drawSprite(gfx_menu_mountains, (u16*)64677, GFX_MENU_MOUNTAINS_W, GFX_MENU_MOUNTAINS_H);


    //
    // DRAW LAKE
    //
    // pattern8 = cpct_px2byteM0(3,3); precalculated pattern8 = 0xcc
    // precalculated: pvideo = cpct_getScreenPtr(CPCT_VMEM_START, 0, 152-3);
    cpct_drawSolidBox ((u16*)60832, 0xcc, 40, 48+3);
    // precalculated: pvideo = cpct_getScreenPtr(CPCT_VMEM_START, 40, 152-3);
    cpct_drawSolidBox ((u16*)60872, 0xcc, 40, 48+3);

}


void drawEnding(){

    cpct_keyID key;
    u8 optionSelected=NO;
    
    g_skyColor=0x5b;
    g_lakeColor=0x57;

    cpct_removeInterruptHandler();                  // Removes Interrupt Handle of the game.
    cpct_setInterruptHandler (interruptHandler);    // Init Interrupt Handle for control.

    cpct_setPalette(ending_palette, 16);    // Set ending palette.
    cpct_clearScreen(0x0c0c);               // clear screen.
    drawLandcape();                         // Draw Baba's Palace and lake.


    // Draw ending text
    wait_n_VSYNCs(25);
    drawText("WELL DONE!!", 30, 5, COLORTXT_ORANGE, DOUBLEHEIGHT, OPAQUE);
    wait_n_VSYNCs(50);
    DrawTextWithCadence("You've defeated all Baba's henchmen and     now the light reigns at Baba's Palace.", 2, 20, COLORTXT_BLUE);
    wait_n_VSYNCs(50);
    DrawTextWithCadence("Baba has fulfilled her promise and reveals to you where to find the elixir of life.", 1, 40, COLORTXT_BLUE);
    wait_n_VSYNCs(50);
    DrawTextWithCadence("What are you waiting for to bring     Little Indian's father back to life?", 8, 60, COLORTXT_MAUVE);


    do {
        key = waitAKey();
        if(key==Key_Esc) playFX(CLICK);
        waitKeyUp(key);

        if(key==Key_Esc) optionSelected=YES;

    } while (optionSelected==NO);

    // Set Black Palette
    cpct_setPalette(game_palette[6], 16);
    g_skyColor=0x54;
    g_lakeColor=0x54;
    // clear Screen
    cpct_clearScreen(0x0c0c);

}

void menuLoop() {
	u8* pvideo, i;
  
    
    drawLandcape(); // Draw Baba's Palace and lake.

    //
	// DRAW STARS
    //
	// pattern8 = cpct_px2byteM0(11,2);
    // pattern8 precalculated 0x8e

	for(i = 0; i < 16; i++) {
        
        if(i<5){    // Big star:

            pvideo = cpct_getScreenPtr(CPCT_VMEM_START, stars[i][0], stars[i][1]);
            cpct_drawSprite(gfx_menu_star_0, pvideo, GFX_MENU_STAR_0_W, GFX_MENU_STAR_0_H);

        } else {    // Little star:

            pvideo = cpct_getScreenPtr(CPCT_VMEM_START, stars[i][0], stars[i][1]);
            // cpct_drawSprite(&pattern8, pvideo, 1, 1);
            *pvideo = 0x8e;
        }
    }

    // Draw title
    drawText("BABA'S PALACE", 28, 5, COLORTXT_ORANGE, DOUBLEHEIGHT, TRANSPARENT);


    // Draw main options.
    drawMainOptions();

    // Draw credits.
    drawText("Code and Gfx      Music and Fx",  10, 160, COLORTXT_BLUE,  NORMALHEIGHT, TRANSPARENT);
    drawText("Azicuetano          McKlain",     12, 170, COLORTXT_BLUE,  NORMALHEIGHT, TRANSPARENT);
    drawText("Game concept by Yutaka Isokawa",  10, 188, COLORTXT_MAUVE, NORMALHEIGHT, TRANSPARENT);


    // To get sincro with raster.
    cpct_waitVSYNC();
    __asm
                halt
                halt
    __endasm;
    cpct_waitVSYNC();


    cpct_setInterruptHandler (interruptHandler);

    fade(IN);
    cpct_setPALColour (16, 0x44);

    // Reset global var to manage menu options.
    g_menuSelectedOption=0;

    cpct_scanKeyboard();
    while (1) {

        checkKeyboard();

        if (g_menuSelectedOption!=0) {
            fade(OUT);
            cpct_removeInterruptHandler();  // Removes Interrupt Handle (no more its needed).
            cpct_akp_stop();                // Stop music.
            return;                         // Start game.
        }
    }
}