//-----------------------------LICENSE NOTICE------------------------------------
//   ____        _           _       _____      _                                
//  |  _ \      | |         ( )     |  __ \    | |                               
//  | |_) | __ _| |__   __ _|/ ___  | |__) |_ _| | __ _  ___ ___                 
//  |  _ < / _` | '_ \ / _` | / __| |  ___/ _` | |/ _` |/ __/ _ |                
//  | |_) | (_| | |_) | (_| | \__ \ | |  | (_| | | (_| | (_|  __/                
//  |____/ \__,_|_.__/ \__,_| |___/ |_|   \__,_|_|\__,_|\___\___|                
//
//  Code & Gfx Copyright (C) 2017 Rafael Castillo (@Azicuetano)
//  Music & Fx Copyright (C) 2017 John McKlain (@elmcklain)
//  This file is part of BABA'S PALACE: an Amstrad CPC Game made with CPCTelera
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include <cpctelera.h>
#include "sprites/portraits.h"
#include "sprites/char.h"
#include "text/text.h"
#include "audio/music.h"
#include "exomizer/exomizer.h"


const u8 full_palette[16] = { 0x55, 0x54, 0x44, 0x55, 0x5c, 0x58, 0x5d, 0x4c, 0x56, 0x5e, 0x40, 0x5f, 0x4e, 0x47, 0x4a, 0x4b };
u8* const portraits[3] = {G_portrait_babayaga, G_portrait_shaolinkid, G_portrait_littleindian };

// Remember the last portrait displayed to not draw it again.
u8 g_oldPortrait;
u8 g_participation;
u8 g_abort;
u16 firmwareMem;


// voice management
const u8 voice[6] = { 0, 8, 12};
u8 actor;

// Declaration
void cpct_scanKeyboard_if();
void cpct_removeInterruptHandler();
void wait_n_VSYNCs(u8 n) __z88dk_fastcall;

////////////////////////////////////////////////////////////////////////////////
//
//    FX SOUNDS MANAGEMENT
//
//    This works by making use of global constants in globals.h that identify each fx sound you want to play.
//    Examples of usage: playFX(SPEAK); playFX(CLICK); ...

#define CLICK     1
#define SPEAK     2

void playFX(u8 fxType) __z88dk_fastcall {

    u8 note=60;

    note += voice[actor];
    note += (cpct_getRandom_lcg_u8 (0)>>7) - 2;

    if(fxType==CLICK) note=48;
    cpct_akp_SFXPlay(fxType, 14, note, 0, 0, AY_CHANNEL_B);
}


////////////////////////////////////////////////////////////////////////////////
//
//    INTERRUPT MANAGEMENT
//
//

void interruptHandler() {
    static u8 k;  // Master interrupt control.

    if (++k == 6) {
        cpct_scanKeyboard_if();
        if (cpct_isKeyPressed(Key_Esc))   {

          while (cpct_isKeyPressed(Key_Esc)) {
            cpct_scanKeyboard_f();
          }

          playFX(CLICK);
          g_abort=1;
        }

        //
        // MUSIC MANAGEMENT (Awesome Music & Fx By McKlain!!!)
        // 
        cpct_akp_musicPlay();
        k = 0;
    }
}

void initCPC() {

  g_oldPortrait=100;  // To force an undefined...
  g_participation=6;  // To manage final pause...
  g_abort=0;          // To abort intro and continue loading the game


  firmwareMem = cpct_disableFirmware();
  cpct_setVideoMode(0);

 
  //  Clear Screen with pen 1(black).
  cpct_waitVSYNC();
  cpct_clearScreen_f64(0xc0c0);

  // Music
  cpct_akp_musicInit(G_music_intro);
  // FX
  cpct_akp_SFXInit (G_fx_intro);

  // set interrupt to play music
  cpct_setInterruptHandler (interruptHandler);

  cpct_setPalette(full_palette, 16);
  //  Pen 16=border, hardware color = 44h = Blue.
  cpct_setPALColour (16, 0x54);
  cpct_waitVSYNC();
}


////////////////////////////////////////////////////////////////////////
// 
// Waits n times for a VSYNC signal (1/50s). After each VSYNC signal,
// if waits for 2 interrupts (using HALT 2 times) to ensure that VSYNC
// stops being active until looking for the next VSYNC signal.
//
// CPCTELERA medium>textscroll example
//
void wait_n_VSYNCs(u8 n) __z88dk_fastcall {
   while(n) {
      if(cpct_isKeyPressed(Key_Space)==1) n=0;
      // Wait for 1 VSYNC signal
      cpct_waitVSYNC();

      // If we still have to wait for more VSYNC signals, wait first
      // for this present VSYNC signal to become inactive. For that,
      // wait for 2 system interrupts using ASM halt instruction.
      if (--n) {
        __asm
          halt
          halt
        __endasm;

      }
   } 
}

//////////////////////////////////////////////////////////////////
// 
// wait4UserKeypress
//    Waits till the user releases a key

void waitKeyUp(cpct_keyID key){

    while (cpct_isKeyPressed(key)) {
    // cpct_scanKeyboard_f();
    }
}

//////////////////////////////////////////////////////////////////
// 
// wait4UserKeypress
//    Waits till the user presses a key, counting the number of
// loop iterations passed.
//
//
// Based on CPCTELERA easy>random example
//
void wait4UserKeypress(u8 posX, u8 posY, u16 textLenght) {
  u32 c = 0;     // Count the number of cycles passed till user k
  static u8 blink = 0;
  u8 *pvmem;

  textLenght<<=3;  // to adjust the time a text is on screen.
  if(g_participation==0) textLenght+=100;

  // Wait 'till the user presses a key, counting loop iterations
  do {
    c++;                       // One more cycle
    
    cpct_waitVSYNC();
    if(c%32==0) {
      blink = !blink; // Blinking cursor state change
      
      if(blink!=0) drawText("_", posX, posY, 0);
      else {
        pvmem = cpct_getScreenPtr(CPCT_VMEM_START, posX, posY+7);
        //precalculated pattern8: 3,3 = 0xcc
        cpct_drawSolidBox (pvmem, 0xcc, 2, 2);
      }

    }

  } while ( !g_abort && cpct_isKeyPressed(Key_Space)==0 && cpct_isKeyPressed(Joy0_Fire1)==0 && c<textLenght); // More or less, textLenght=384 --> wait 6.5 seconds or a key press.

  playFX(CLICK);
  wait_n_VSYNCs(10);
}

#define EXOBUFFER_ADDRESS 0x6000
void drawPortrait(u8 nPortrait) {

  u8* pvmem;  // Pointer to video memory
  
  decompress(portraits[nPortrait], (u8*)EXOBUFFER_ADDRESS);
  
  pvmem = cpct_getScreenPtr(CPCT_VMEM_START, 10, 28);
  cpct_drawSprite ((u8*)EXOBUFFER_ADDRESS, pvmem, 16, 64);

}


// All pareters are precalculated for increase draw speed and save some bytes.
void drawTextWindow(){

  cpct_waitVSYNC ();

  // -- WHITELINES: precalculated pattern color (15,15) = 0xff

  //pvmem = cpct_getScreenPtr(CPCT_VMEM_START, 5, 94)   --> pvmem = (u16*)0xF375; //62325;
  cpct_drawSolidBox ((u16*)0xF375, 0xff, 35, 2);
  //pvmem = cpct_getScreenPtr(CPCT_VMEM_START, 40, 94) --> pvmem = (u16*)0xF398; //62360;
  cpct_drawSolidBox ((u16*)0xF398, 0xff, 35, 2);

  //pvmem = cpct_getScreenPtr(CPCT_VMEM_START, 5, 152) --> pvmem = (u16*)0xC5F5; // 50677;
  cpct_drawSolidBox ((u16*)0xC5F5, 0xff, 35, 2);
  //pvmem = cpct_getScreenPtr(CPCT_VMEM_START, 40, 152)--> pvmem = (u16*)0xC618; // 50712;
  cpct_drawSolidBox ((u16*)0xC618, 0xff, 35, 2);


  // -- BLUEBOX: precalculated pattern color (3,3) = 0xcc

  //pvmem = cpct_getScreenPtr(CPCT_VMEM_START, 5, 98); --> pvmem = (u16*)0xD3C5; // 54213;
  cpct_drawSolidBox ((u16*)0xD3C5, 0xcc, 35, 52);
  //pvmem = cpct_getScreenPtr(CPCT_VMEM_START, 40, 98);--> pvmem = (u16*)0xD3E8; // 54248;
  cpct_drawSolidBox ((u16*)0xD3E8, 0xcc, 35, 52);


  // -- LEFT & RIGHT BORDERS: precalculated pattern (15, 1) = 0xea, (15,1) = 0xd5
  
  // left vertical line
  //pvmem = cpct_getScreenPtr(CPCT_VMEM_START, 4, 98)  --> pvmem = (u16*)0xD3C4; // 54212;
  cpct_drawSolidBox ((u16*)0xD3C4, 0xea, 1, 52);
    
  // top right corner
  //pvmem = cpct_getScreenPtr(CPCT_VMEM_START, 75, 96) --> pvmem = (u16*)0xC40B; // 50187;
  cpct_drawSolidBox ((u16*)0xC40B, 0xea, 1, 2);
  
  // bottom right corner
  //pvmem = cpct_getScreenPtr(CPCT_VMEM_START, 75, 150)--> pvmem = (u16*)0xF5EB; // 62955;
  cpct_drawSolidBox ((u16*)0xF5EB, 0xea, 1, 2);

  
  // right vertical line
  //pvmem = cpct_getScreenPtr(CPCT_VMEM_START, 75, 98) --> pvmem = (u16*)0xD40B; // 54283;
  cpct_drawSolidBox ((u16*)0xD40B, 0xd5, 1, 52);

  // top left corner
  //pvmem = cpct_getScreenPtr(CPCT_VMEM_START, 4, 96)  --> pvmem = (u16*)0xC3C4; // 50116;
  cpct_drawSolidBox ((u16*)0xC3C4, 0xd5, 1, 2);

  //bottom left corner
  //pvmem = cpct_getScreenPtr(CPCT_VMEM_START, 4, 150) --> pvmem = (u16*)0xF5A4; // 62884;
  cpct_drawSolidBox ((u16*)0xF5A4, 0xd5, 1, 2);

}

#define INIT_X_BOXTEXT      6
#define INIT_Y_BOXTEXT      103
#define LINEHEIGHT          11

#define BOXTEXT_HISTART     98
#define BOXTEXT_HIHEIGHT    52
#define BOXTEXT_LOWSTART    113
#define BOXTEXT_LOWHEIGHT   37


void clearText(u8 boxtextStart, u8 boxtextHeight){
  u8 *pvmem;

  // Clear text
  // precalculated patter8 = 0xcc
  // pattern8 = cpct_px2byteM0(3,3);
  pvmem = cpct_getScreenPtr(CPCT_VMEM_START, 5, boxtextStart);
  cpct_drawSolidBox (pvmem, 0xcc, 35, boxtextHeight);
  pvmem = cpct_getScreenPtr(CPCT_VMEM_START, 40, boxtextStart);
  cpct_drawSolidBox (pvmem, 0xcc, 35, boxtextHeight);
}

void messageBox(u8 portrait, u8 *texto){
  u8 i=0, offsetX=0, offsetY=LINEHEIGHT, originX=6, originY=103, character[]="_";
  const u8 *name[3]={"Baba Yaga:", "Shaolin Kid:", "Little Indian:"};
  u8 boxtextStart=BOXTEXT_LOWSTART, boxtextHeight=BOXTEXT_LOWHEIGHT;

  // Ugly trick to abort intro... God forgive me...
  if(g_abort) return;

  --g_participation;

  // DRAWING PORTRAIT PICTURES AND NAMES
  if(portrait!=g_oldPortrait) {
    g_oldPortrait=portrait;
    clearText(BOXTEXT_HISTART, BOXTEXT_HIHEIGHT);
    

    drawPortrait(portrait);
    drawText(name[portrait], 6, 103, 1);


  } else clearText(BOXTEXT_LOWSTART, BOXTEXT_LOWHEIGHT);
          

    // WRITING TEXT WITH CADENCE
    actor=portrait;

    while (character[0] != '\0' && !g_abort) {      

      // Store the caracter.
      character[0] = texto[i];
      if(offsetX==0 && character[0] == ' ') offsetX-=2;

      if(character[0] == ' ') {
        playFX(SPEAK);
        wait_n_VSYNCs(2);
      }
      //cpct_waitVSYNC ();
      wait_n_VSYNCs(1);

      drawText(character, offsetX+originX, offsetY+originY, 0);

      // Performs a dramatic pause
      if(character[0] == '.' || character[0] == ',' || character[0] == '!' ) wait_n_VSYNCs(25);
      
      // Correction to support narrow wide chars, like "i", "l" and "'".
      if(character[0] == 'i' || character[0] == 'l' || character[0] == '\'') --offsetX;

      offsetX+=2;
      if(offsetX>66) {
        offsetX=0;
        offsetY+=LINEHEIGHT;
      }

      i++;
    }
    // playFX(SPEAK);
    if(!g_abort) wait4UserKeypress(offsetX+originX-2, offsetY+originY, i);
}

void storytelling(){

  drawTextWindow();

  // ':' prints a '.' without dramatic pause.
  messageBox(1, "Hi Baba, we've heard that you are a very powerful witch. We need you to help us with your powers:");
  messageBox(2, "We need the elixir of life to bring  my father back to life. Please, can you tell us where to find it?");
  messageBox(0, "Do you think I'll give you the info  you're looking for without asking   for anything in return?");
  messageBox(0, "Hahaha! Please, I am a true witch, not a fairy:");
  messageBox(0, "If you want to know where is the  elixir of life, you will have to      defeat all my henchmen:::");
  messageBox(0, "and descend into the depths of my palace. You have one hundred levels ahead! Are you ready?");

}

void main(void) {

  initCPC();

  wait_n_VSYNCs(60);
  storytelling();

  cpct_clearScreen_f64(0xc0c0);         // Clear Screen with pen 1(black).
  cpct_removeInterruptHandler();        // Removes Interrupt Handle (no more its needed).
  cpct_akp_stop();                      // Stop music.
  cpct_reenableFirmware(firmwareMem);   // reenable firmware to continue with laod screen.
}